import java.util.*;

/**
 * Personenliste mit Funktionen zur Berechnung
 * verschiedener Eigenschaften der Personen in der Liste
 *
 * @author Bernhard Fiser 0109815
 * @version 2.0
 */

public class PersonenVerwaltung
{
	LinkedList	l;
	Person		p;
	private int	a;
	private int	c;
	private ListIterator
			li;

	/** Initialisiert Personenliste */
	public		PersonenVerwaltung(LinkedList list)
	{
		this.l = list;
	}

	/** Ermittelt die Anzahl der Personen in der Liste
	 * @param int type: 0 = alle, 1 = Arbeiter, 2 = Angestellter
	 */
	public int	size(int type)
	{
		int	i = 0;

		if (type == 0)
			return l.size();

		li = l.listIterator(0);
		while (li.hasNext())
		{
			p = (Person) li.next();
			if (((type == 1) && (p instanceof Angestellter)) || ((type == 2) && (p instanceof Arbeiter)))
				continue;
			i++;
		}
		return(i);
	}


	/** Ermittelt das Durchschnittsalter aller Personen
	 * @param int type: 0 = alle, 1 = Arbeiter, 2 = Angestellter
	 */
	public int	avg_age(int type)
	{
		/* Listeniterator initialisieren und auf den Anfang der Liste setzen */
		li = l.listIterator(0);
		a = 0;
		c = 0;

		/* Liste vom ersten bis zum letzten Element traversieren */
		while (li.hasNext())
		{
			/* Aktuelles Personenelemt auslesen */
			p = (Person) li.next();

			if (((type == 1) && (p instanceof Angestellter)) || ((type == 2) && (p instanceof Arbeiter)))
				continue;
			/* Durchschnitt berechnen falls nicht erstes Element
			 * (oder Person 0 Jahre alt) */
			a += p.getAlter();
			c++;
		}
		return a / c;
	}

	/** Ermittelt das Alter der jngsten Person
	 * @param int type: 0 = alle, 1 = Arbeiter, 2 = Angestellte
	 */
	public int	min_age(int type)
	{
		li = l.listIterator(0);
		a = 1000;

		while (li.hasNext())
		{
			p = (Person) li.next();
			if (((type == 1) && (p instanceof Angestellter)) || ((type == 2) && (p instanceof Arbeiter)))
				continue;
			/* Falls Person jnger als vorhergehende Person,
			 * Alter merken */
			if (a > p.getAlter())
				a = p.getAlter();
		}
		return a;
	}

	/** Ermittelt das Alter der ltesten Person
	 * @param int type: 0 = alle, 1 = Arbeiter, 2 = Angestellte
	 */
	public int	max_age(int type)
	{
		li = l.listIterator(0);
		a = 0;
		while (li.hasNext())
		{
			p = (Person) li.next();
			if (((type == 1) && (p instanceof Angestellter)) || ((type == 2) && (p instanceof Arbeiter)))
				continue;
			/* Falls Person lter als vorhergehende Person,
			 * Alter merken */
			if (a < p.getAlter())
				a = p.getAlter();
		}
		return a;
	}

	/** Listet alle Personen auf die jnger sind als der
	 * angegebene Schwellwert <avg>
	 * @param int age: Alter
	 * @param int type: 0 = alle, 1 = Arbeiter, 2 = Angestellte
	 */
	public int	below_avg(int avg, int type)
	{
		li = l.listIterator(0);
		while (li.hasNext())
		{
			p = (Person) li.next();
			if (((type == 1) && (p instanceof Angestellter)) || ((type == 2) && (p instanceof Arbeiter)))
				continue;
			/* Falls Person jnger als <avg>, dann Person
			 * auf stdout ausgeben */
			if (p.getAlter() <= avg)
				p.print();
		}
		return a;
	}

	/**
	 * Person mit bestimmtem Alter und Typ ausgeben.
	 * @param int age: Alter
	 * @param int type: 0 = alle, 1 = Arbeiter, 2 = Angestellte
	 */
	public void	print_age(int age, int type)
	{
		li = l.listIterator(0);
		while (li.hasNext())
		{
			p = (Person) li.next();
			if (((type == 1) && (p instanceof Angestellter)) || ((type == 2) && (p instanceof Arbeiter)))
				continue;
			if (p.getAlter() == age)
				p.print();
		}
	}

	/** Listet alle Personen auf, die lter sind als
	 * der angegebene Schwellwert <avg>
	 * @param int avg: Durchschnittsgehalt
	 * @param int type: 0 = alle, 1 = Arbeiter, 2 = Angestellte
	 */
	public int	above_avg(int avg, int type)
	{
		li = l.listIterator(0);
		while (li.hasNext())
		{
			p = (Person) li.next();
			if (((type == 1) && (p instanceof Angestellter)) || ((type == 2) && (p instanceof Arbeiter)))
				continue;
			/* Falls Person lter als <avg>, dann Person
			 * auf stdout ausgeben */
			if (p.getAlter() > avg)
				p.print();
		}
		return a;
	}

	/** Ermittelt das Durchschnittsgehalt aller Personen
	 * @param int type: 0 = alle, 1 = Arbeiter, 2 = Angestellter
	 */
	public int	avg_gehalt(int type)
	{
		/* Listeniterator initialisieren und auf den Anfang der Liste setzen */
		li = l.listIterator(0);
		a = 0;
		c = 0;

		/* Liste vom ersten bis zum letzten Element traversieren */
		while (li.hasNext())
		{
			/* Aktuelles Personenelemt auslesen */
			p = (Person) li.next();

			if (((type == 1) && (p instanceof Angestellter)) || ((type == 2) && (p instanceof Arbeiter)))
				continue;
			/* Durchschnitt berechnen falls nicht erstes Element
			 * (oder Person 0 Jahre alt) */
			a += p.getGehalt();
			c++;
		}
		return a / c;
	}

	/**
	 * Anzahl der Personen die unter dem Durchschnittsgehalt liegen
	 * @param int avg: Durchschnittsgehalt
	 * @param int type: 0 = alle, 1 = Arbeiter, 2 = Angestellte
	 */
	public int	below_avg_gehalt(int avg, int type)
	{
		a = 0;
		li = l.listIterator(0);
		while (li.hasNext())
		{
			p = (Person) li.next();
			if (((type == 1) && (p instanceof Angestellter)) || ((type == 2) && (p instanceof Arbeiter)))
				continue;
			/* Falls Person jnger als <avg>, dann Person
			 * auf stdout ausgeben */
			if (p.getGehalt() <= avg)
				a++;
		}
		return a;
	}

}

