import java.io.*;
import java.util.*;


/** Hauptprogramm zur Personenverwaltung.
 * Liest Personendaten aus Textfile ein,
 * hngt diese in verkettete Liste und gibt statistische
 * Daten ber die Personen aus.
 *
 * @author Bernhard Fiser
 * @version 1.0
 */

class TestPersonenVerwaltung
{
	static PersonenFileReader
				arbf, angf;
	static LinkedList	bh;
	static PersonenVerwaltung
				pl;
	static int		avg;

	public static void main(String argv[])
	{
		int		i;
		String[]	stype = {"Personen", "Arbeiter", "Angestellte"};

		/* Neuen PersonenFileReader anlegen, dient
		 * zum einlesen des Personenfiles */
		try
		{
			arbf = new ArbeiterFileReader(argv[0]);
			angf = new AngestelltenFileReader(argv[1]);
		}
		catch (ArrayIndexOutOfBoundsException e)
		{
			System.out.println("*** usage: java TestPersonenVerwaltung <Arbeiter filename> <Angestellten filename>");
			return;
		}

		/* Personenfile ffnen */
      		try
		{
			arbf.open();
			angf.open();
		}
		catch (IOException e) { System.out.println("*** error while reading from file"); return; }

		Person pers = null;

		try
		{
			/* Neues LinkedList-Object erstellen */
			bh = new LinkedList();

			/* Personen aus File auslesen und in verkettete Liste
			 * aufnehmen */
			while ((pers = arbf.readNext()) != null)
				bh.add(pers);
			while ((pers = angf.readNext()) != null)
				bh.add(pers);
		}

		catch (IOException e) { System.out.println("*** Error in file\n"); }

		/* Personenfile schliessen */
		try
		{
			arbf.close();
			angf.close();
		}

		catch (IOException e) { System.out.println("*** Error closing file\n"); }

		/* Neues Personenlistenobject erstellen */
		pl = new PersonenVerwaltung(bh);

		/* Statistische Daten ermitteln und auf stdout ausgeben */
		for (i = 0; i < 3; i++)
		{
			System.out.println("**************** " + stype[i] + " ****************");
			System.out.println("Anzahl " + stype[i] + ": " + pl.size(i));
			System.out.println("Durchschnittsalter " + stype[i] + ": " + pl.avg_age(i));
			System.out.println("Durchschnittsgehalt " + stype[i] + ": " + pl.avg_gehalt(i));
			System.out.println("Juengste aller " + stype[i] + ": " + pl.min_age(i));
			pl.print_age(pl.min_age(i), i);
			System.out.println("Alteste aller " + stype[i] + ": " + pl.max_age(i));
			pl.print_age(pl.max_age(i), i);
			System.out.println(stype[i] + " ueber dem Durchschnitt:");
			pl.above_avg(pl.avg_age(i), i);
			System.out.println(stype[i] + " unter dem Durchschnitt:");
			pl.below_avg(pl.avg_age(i), i);
			System.out.println(stype[i] + "gehalt unter dem Durchschnitt: " + pl.below_avg_gehalt(pl.avg_gehalt(i), i));
		}
	}
}
